package core;

import java.util.List;
public class DeepEnergyModule {
    public static final String HAS_ENERGY_S = "energyEnabled";
    public static final String MAX_CAPACITY_S = "energyMaxCapacity";
    public static final String INITIAL_CHARGE_S = "energyInitialCharge";
    public static final String USAGE_RATE_MOVEMENT_S =
            "energyUsageRateMovement";
    public static final String CHARGE_RATE_SLOW_S = "energyChargeRateSlow";
    public static final String CHARGE_RATE_FAST_S = "energyChargeRateFast";
    public static final String REFUEL_NEED_THRESHOLD = "refuelNeedThreshold";
    public static final String REFUEL_COMPLETE_THRESHOLD =
            "refuelCompleteThreshold";
    public static final String ENERGY_TRANSFER_RANGE_S = "energyTransferRange";
    public static final String ENERGY_OASIS_S = "energyOasis";
    public static final double DEFAULT_MAX_CAPACITY = 17.0;
    public static final double DEFAULT_INITIAL_CHARGE = DEFAULT_MAX_CAPACITY;
    public static final double MIN_CAPACITY = 0;
    public static final double DEFAULT_USAGE_RATE_MOVEMENT = 0.0003;
    public static final double DEFAULT_CHARGE_RATE_SLOW = 0.00036;
    public static final double DEFAULT_CHARGE_RATE_FAST = 0.0015;
    public static final double DEFAULT_REFUEL_NEED_THRESHOLD = 25.0;
    public static final double DEFAULT_REFUEL_COMPLETE_THRESHOLD = 75.0;
    public static final double DEFAULT_ENERGY_TRANSFER_RANGE = 10.0;
    public static final boolean DEFAULT_ENERGY_OASIS_STATUS = false;

    private final double maxCapacity;
    private final double usageRateMovement;
    private double remainingCapacity;
    private double chargeRateSlow;
    private double chargeRateFast;
    private double refuelNeedThreshold;
    private double refuelCompleteThreshold;
    private double energyTransferRange;
    private boolean isEnergyOasis;
    private List<DTNHost> connections;
    private DTNHost host;
    public DeepEnergyModule() {
        this(DEFAULT_MAX_CAPACITY, DEFAULT_INITIAL_CHARGE,
                DEFAULT_USAGE_RATE_MOVEMENT, DEFAULT_CHARGE_RATE_SLOW,
                DEFAULT_CHARGE_RATE_FAST, DEFAULT_REFUEL_NEED_THRESHOLD,
                DEFAULT_REFUEL_COMPLETE_THRESHOLD,
                DEFAULT_ENERGY_TRANSFER_RANGE, DEFAULT_ENERGY_OASIS_STATUS);
    }
    DeepEnergyModule(double maxCapacity, double remainingCapacity, double
            usageRateMovement, double chargeRateSlow, double chargeRateFast,
                     double refuelNeedThreshold, double refuelCompleteThreshold,
                     double energyTransferRange, boolean oasis) {
        this.maxCapacity = maxCapacity;
        this.remainingCapacity = remainingCapacity;
        this.usageRateMovement = usageRateMovement;
        this.chargeRateSlow = chargeRateSlow;
        this.chargeRateFast = chargeRateFast;
        this.refuelNeedThreshold = refuelNeedThreshold;
        this.refuelCompleteThreshold = refuelCompleteThreshold;
        this.energyTransferRange = energyTransferRange;
        this.isEnergyOasis = oasis;
    }
    private DeepEnergyModule(DeepEnergyModule prototype) {
        this.maxCapacity = prototype.maxCapacity;
        this.usageRateMovement = prototype.usageRateMovement;
        this.remainingCapacity = prototype.remainingCapacity;
        this.chargeRateSlow = prototype.chargeRateSlow;
        this.chargeRateFast = prototype.chargeRateFast;
        this.refuelNeedThreshold = prototype.refuelNeedThreshold;
        this.refuelCompleteThreshold = prototype.refuelCompleteThreshold;
        this.energyTransferRange = prototype.energyTransferRange;
        this.isEnergyOasis = prototype.isEnergyOasis;
    }
    public void chargeSlow(double duration) {
        this.remainingCapacity += duration * this.chargeRateSlow;
        if (this.remainingCapacity > this.maxCapacity) {
            this.remainingCapacity = this.maxCapacity;
        }
    }
    public void chargeFast(double duration) {
        this.remainingCapacity += duration * this.chargeRateFast;
        if (this.remainingCapacity > this.maxCapacity) {
            this.remainingCapacity = this.maxCapacity;
        }
    }
    public void discharge(double duration) {
        this.remainingCapacity -= duration * this.chargeRateSlow;
        if (!this.isEnergyOasis && this.remainingCapacity < 0) {
            this.remainingCapacity = 0;
        }
    }
    public boolean drainViaMovement(double distance) {
        double futureCapacity = this.remainingCapacity - (usageRateMovement
                * distance);        if (futureCapacity <= MIN_CAPACITY) {
            this.remainingCapacity = MIN_CAPACITY;
            return false;
        } else {
            this.remainingCapacity = futureCapacity;
            return true;
        }
    }
    public DeepEnergyModule replicate() {
        return new DeepEnergyModule(this);
    }
    public double getRemainingCapacity() {
        return this.remainingCapacity;
    }
    public double remainingCapacityPercentage() {
        return this.remainingCapacity / (this.maxCapacity / 100);
    }
    public boolean isInNeedOfFuel() {
        return this.remainingCapacityPercentage() <= this.refuelNeedThreshold;
    }
    public double getRefuelNeedThreshold() {
        return refuelNeedThreshold;
    }
    public double getRefuelCompleteThreshold() {
        return refuelCompleteThreshold;
    }
    public double getEnergyTransferRange() {
        return energyTransferRange;
    }
    public synchronized void addEnergyContact(DTNHost host) {
        this.connections.add(host);
    }
    public void resetEnergyContacts() {
        this.connections.clear();
    }
    public List<DTNHost> getEnergyContacts() {
        return this.connections;
    }
    public boolean isEnergyOasis() {
        return this.isEnergyOasis;
    }
    public boolean isDepleted() {
        if (this.isEnergyOasis) {
            return false;
        } else if (this.remainingCapacityPercentage() > 0) {
            return false;
        }
        return true;
    }
}
