package routing.util;
public abstract class EnergyModule {
    protected static final int CAPACITY_MAX = 100;
    protected static final int CAPACITY_MIN = 0;
    protected Double[] chargeRates, dischargeRates;
    protected double remainingCapacity;
    public EnergyModule(double capacityKwh, Double[] chargeTimesHours,
                        Double[] dischargeTimesHours) {
        if (chargeTimesHours == null || chargeTimesHours.length == 0 ||
                dischargeTimesHours == null || dischargeTimesHours.length ==
                0) {
            System.out.println("EnergyModule: initialising with invalid " +
                    "charge/discharge times");
            System.exit(1);
        }
        this.chargeRates = new Double[chargeTimesHours.length];
        this.dischargeRates = new Double[dischargeTimesHours.length];
        for (int i = 0; i < chargeTimesHours.length; i++) {
            Double time = chargeTimesHours[i];
            if (time <= 0) {
                System.out.println("EnergyModule: can't have charge time" +
                        " < 0");
                System.exit(1);
            } else {
                this.chargeRates[i] = EnergyModule.energyRateToPercentage
                        (capacityKwh, time);
            }
        }
        for (int i = 0; i < dischargeTimesHours.length; i++) {
            Double time = dischargeTimesHours[i];
            if (time <= 0) {
                System.out.println("DeepEnergyModule: can't have charge time" +
                        " < 0");
                System.exit(1);
            } else {
                this.dischargeRates[i] = EnergyModule.energyRateToPercentage
                        (capacityKwh, time);
            }
        }
        this.remainingCapacity = 100;
    }
    public void charge(int rate, double interval) {
        double newCapacity;
        if (rate < 0 || rate > this.chargeRates.length - 1) {
            System.out.println("EnergyModule: attempting charging at an " +
                    "" + "unspecified rate");
            System.exit(1);
        }
        newCapacity = this.remainingCapacity + this.chargeRates[rate] *
                interval;
        if (newCapacity > CAPACITY_MAX) {
            this.remainingCapacity = CAPACITY_MAX;
        } else {
            this.remainingCapacity = newCapacity;
        }
    }
    public void discharge(int rate, double interval) {
        double newCapacity;
        if (rate < 0 || rate > this.dischargeRates.length - 1) {
            System.out.println("EnergyModule: attempting discharging at " +
                    "an " + "unspecified rate");
            System.exit(1);
        }
        newCapacity = this.remainingCapacity - this.dischargeRates[rate] *
                interval;
        if (newCapacity < 0) {
            this.remainingCapacity = 0;
        } else {
            this.remainingCapacity = newCapacity;
        }
    }
    public double getRemainingCapacity() {
        return this.remainingCapacity;
    }
    private static double energyRateToPercentage(double capacityKwh, double
            hoursToCharge) {
        double kwPerSecond = capacityKwh / (hoursToCharge * 3600);
        return (kwPerSecond / capacityKwh) * 100;
    }
    public void deplete() {
        this.remainingCapacity = 0.0;
    }
}
