package routing;
import core.Settings;
import org.apache.commons.lang3.tuple.Pair;
import routing.util.EnergyModule;
import routing.util.EnergySeeker;
import routing.util.VehicleBattery;
import java.util.Collections;
import java.util.Comparator;
public class EnergySeekerRouter extends EnergyRouter implements EnergySeeker {
    public EnergySeekerRouter(Settings settings) {
        super(settings);
        this.state = State.NEUTRAL;
        this.energyModule = new VehicleBattery();
    }
    public EnergySeekerRouter(EnergySeekerRouter prototype) {
        super(prototype);
        this.state = prototype.state;
        this.energyModule = new VehicleBattery();
    }
    @Override
    public MessageRouter replicate() {
        return new EnergySeekerRouter(this);
    }
    @Override
    protected void updateState() {
        if (this.energyModule.getRemainingCapacity() < 70) {
            this.state = State.SEEK;
        } else {
            this.state = State.NEUTRAL;
        }
    }
    @Override
    protected Contact pickDesiredSupplier() {
        Contact chosenSupplier = null;
        double minTime = Double.MAX_VALUE;
        for (Contact supplier : this.suppliers) {
            if (supplier.queue.size() - supplier.plugCount < 0) {
                return supplier;
            }
            for (Pair<String, Double> p : supplier.queue) {
                double time = p.getRight();
                if (time < minTime) {
                    minTime = time;
                    chosenSupplier = supplier;
                }
            }
        }
        return chosenSupplier;
    }

    @Override
    protected void orderSeekers() {
        System.out.println(
                "EnergySeekerRouter: attempting to order seekers but" +
                        "seeker/neutral node is never a supplier");
        System.exit(1);
    }
    @Override
    protected void chargePeer(EnergySeeker seeker, double interval) {
        System.out.println("EnergySeekerRouter: attempting to charge a peer" +
                "but node is never a supplier");
        System.exit(1);
    }
    @Override
    public boolean isBelowThreshold() {
        return this.energyModule.getRemainingCapacity() < 40;
    }
    @Override
    public EnergyModule getEnergyModule() {
        return this.energyModule;
    }
}
